import logging
import json
from glob import glob
import os
import httpx
import re


def link_upload(local_paths, link, password=None):
    "use to upload to an upload link generated by seafile"
    if password:
        raise NotImplementedError("Passwords are not implemented yet")
    if not link.endswith("/"):
        link += "/"
    body = httpx.get(link).text

    library_id = re.search(r"r=(\S+)'", body, re.MULTILINE).groups()[0]
    parent_dir = re.search(r"parent_dir': \"(\S+)\".*", body, re.MULTILINE).groups()[0]
    parts = link.split("/")
    ajax_url = "/".join(parts[0:3] + ["ajax"] + parts[3:-1] + ["upload"] + [""])
    if isinstance(local_paths, str):
        local_paths = [local_paths]
    for glob_path in local_paths:
        glob_prefix = glob_path.split("*")[0]

        for path in glob(glob_path, recursive=True):
            if os.path.isfile(path):
                resp = httpx.get(
                    ajax_url,
                    params={"r": library_id},
                    headers={
                        "Accept": "application/json",
                        "X-Requested-With": "XMLHttpRequest",
                    },
                )
                upload_url = json.loads(resp.text).get("url")
                with open(path, "rb") as fh:
                    if glob_prefix != path:
                        pdir = (
                            parent_dir + os.path.dirname(path)[len(glob_prefix) :] + "/"
                        )
                    else:
                        pdir = parent_dir
                    fn = os.path.basename(path)
                    httpx.post(
                        upload_url,
                        data={"parent_dir": pdir, "fn": fn, "repo_id": library_id},
                        files={"file": (fn, fh, "application/octet-stream")},
                    )
